
From: "Andi Kleen" <ak@suse.de>

Add SLIT (inter node distance) information to sysfs. 

[This is Jack's patch that he submitted on l-k. I'm submitting
it for him because I need it for my x86-64 followon SLIT patch.
Hope I don't stomp onto his toes with that one. If you already 
merged it please ignore]

From: Jack Steiner

Here is an update patch to externalize the SLIT information. I think I have
encorporated all the comments that were posted previously)

For example:

        # cd /sys/devices/system
        # find .
        ./node
        ./node/node5
        ./node/node5/distance
        ./node/node5/numastat
        ./node/node5/meminfo
        ./node/node5/cpumap

        # cat ./node/node0/distance
        10 20 64 42 42 22

        # cat node/*/distance
        10 20 64 42 42 22
        20 10 42 22 64 84
        64 42 10 20 22 42
        42 22 20 10 42 62
        42 64 22 42 10 20
        22 84 42 62 20 10

Signed-off-by: Jack Steiner <steiner@sgi.com>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/drivers/base/node.c         |   19 +++++++++++++++++++
 25-akpm/include/asm-i386/topology.h |    3 ---
 25-akpm/include/linux/topology.h    |    5 ++++-
 3 files changed, 23 insertions(+), 4 deletions(-)

diff -puN drivers/base/node.c~x86_64-add-slit-inter-node-distance-information-to drivers/base/node.c
--- 25/drivers/base/node.c~x86_64-add-slit-inter-node-distance-information-to	2004-11-28 01:54:51.115826320 -0800
+++ 25-akpm/drivers/base/node.c	2004-11-28 01:54:51.122825256 -0800
@@ -111,6 +111,24 @@ static ssize_t node_read_numastat(struct
 }
 static SYSDEV_ATTR(numastat, S_IRUGO, node_read_numastat, NULL);
 
+static ssize_t node_read_distance(struct sys_device * dev, char * buf)
+{
+	int nid = dev->id;
+	int len = 0;
+	int i;
+
+	/* buf currently PAGE_SIZE, need ~4 chars per node */
+	BUILD_BUG_ON(NR_NODES*4 > PAGE_SIZE/2);
+
+	for (i = 0; i < numnodes; i++)
+		len += sprintf(buf + len, "%s%d", i ? " " : "", node_distance(nid, i));
+
+	len += sprintf(buf + len, "\n");
+	return len;
+}
+static SYSDEV_ATTR(distance, S_IRUGO, node_read_distance, NULL);
+
+
 /*
  * register_node - Setup a driverfs device for a node.
  * @num - Node number to use when creating the device.
@@ -129,6 +147,7 @@ int __init register_node(struct node *no
 		sysdev_create_file(&node->sysdev, &attr_cpumap);
 		sysdev_create_file(&node->sysdev, &attr_meminfo);
 		sysdev_create_file(&node->sysdev, &attr_numastat);
+		sysdev_create_file(&node->sysdev, &attr_distance);
 	}
 	return error;
 }
diff -puN include/asm-i386/topology.h~x86_64-add-slit-inter-node-distance-information-to include/asm-i386/topology.h
--- 25/include/asm-i386/topology.h~x86_64-add-slit-inter-node-distance-information-to	2004-11-28 01:54:51.117826016 -0800
+++ 25-akpm/include/asm-i386/topology.h	2004-11-28 01:54:51.123825104 -0800
@@ -66,9 +66,6 @@ static inline cpumask_t pcibus_to_cpumas
 	return node_to_cpumask(mp_bus_id_to_node[bus]);
 }
 
-/* Node-to-Node distance */
-#define node_distance(from, to) ((from) != (to))
-
 /* sched_domains SD_NODE_INIT for NUMAQ machines */
 #define SD_NODE_INIT (struct sched_domain) {		\
 	.span			= CPU_MASK_NONE,	\
diff -puN include/linux/topology.h~x86_64-add-slit-inter-node-distance-information-to include/linux/topology.h
--- 25/include/linux/topology.h~x86_64-add-slit-inter-node-distance-information-to	2004-11-28 01:54:51.118825864 -0800
+++ 25-akpm/include/linux/topology.h	2004-11-28 01:54:51.123825104 -0800
@@ -55,7 +55,10 @@ static inline int __next_node_with_cpus(
 	for (node = 0; node < numnodes; node = __next_node_with_cpus(node))
 
 #ifndef node_distance
-#define node_distance(from,to)	((from) != (to))
+/* Conform to ACPI 2.0 SLIT distance definitions */
+#define LOCAL_DISTANCE		10
+#define REMOTE_DISTANCE		20
+#define node_distance(from,to)	((from) == (to) ? LOCAL_DISTANCE : REMOTE_DISTANCE)
 #endif
 #ifndef PENALTY_FOR_NODE_WITH_CPUS
 #define PENALTY_FOR_NODE_WITH_CPUS	(1)
_
